import { ɵɵdefineInjectable, Injectable, EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import { HttpEventType, HttpClient } from '@angular/common/http';
import { CommonModule } from '@angular/common';

class NgUploadImageService {
    constructor() { }
}
NgUploadImageService.ɵprov = ɵɵdefineInjectable({ factory: function NgUploadImageService_Factory() { return new NgUploadImageService(); }, token: NgUploadImageService, providedIn: "root" });
NgUploadImageService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgUploadImageService.ctorParameters = () => [];

class NgUploadImageComponent {
    constructor(http) {
        this.http = http;
        this.selectedUrls = [];
        this.selectedFiles = [];
        this.uploadProgress = 0;
        this.validTypes = ["image/png", "image/jpeg"];
        this.maxUploadCount = 12;
        this.preViewSize = 100;
        this.apiUrl = "";
        this.onResponse = new EventEmitter();
    }
    ngOnInit() {
        this.isValidDevInput();
    }
    /* ********************************* */
    /** //////////  validate  \\\\\\\\\\ */
    /* ********************************* */
    /**
     * @property {Function} isValidType - validate developper  @Input()
     * @returns {void} - is valid or not.
     */
    isValidDevInput() {
        this.apiUrl ? {} : this.setError("api url not defined");
        // throw new Error('Function not implemented.');
    }
    /**
     * @property {Function} isValidType - set valid image type
     * @param {type} - image type
     * @returns {boolean} - is valid or not.
     */
    isValidType(type) {
        return this.validTypes.includes(type);
    }
    /**
     * @property {Function} isMaxUploadCount - set valid image type
     * @returns {boolean} - is valid or not.
     */
    isMaxUploadCount() {
        return this.selectedFiles.length <= this.maxUploadCount;
    }
    /* ********************************* */
    /** ////////// user events \\\\\\\\\\ */
    /* ********************************* */
    /**
     * @property {Function} onSelect - select image to upload list
     * @param {MouseEvent} - select event
     * @returns {void}
     */
    onSelect(event) {
        const files = event.target['files'];
        for (let file of files) {
            if (this.isValidType(file.type)) {
                const reader = new FileReader();
                reader.readAsDataURL(file);
                reader.onload = (evnets) => {
                    this.selectedUrls.push(evnets.target.result);
                    this.selectedFiles.push(file);
                };
            }
            else {
                console.log("Warning : ", "This image format not allow");
            }
        }
    }
    /**
     * @property {Function} onUpload - start upload to serve
     * @param {MouseEvent} - click event
     * @returns {void}
     */
    onUpload(event) {
        if (this.isMaxUploadCount()) {
            const fd = new FormData();
            for (let file of this.selectedFiles) {
                fd.append('images', file, file['name']);
            }
            //fd.append('datax','cc');
            this.http.post(this.apiUrl, fd, {
                reportProgress: true,
                observe: 'events'
            }).subscribe(event => {
                if (event.type === HttpEventType.UploadProgress) {
                    this.setUploadProgress(event.loaded / event.total);
                }
                else if (event.type === HttpEventType.Response) {
                    const res = event['body'];
                    this.setUploadResponse(event);
                    if (res && res['success']) {
                        this.setUploadProgress(0);
                        this.setClearSelected();
                    }
                    else {
                        console.log('Error', res['message']);
                    }
                }
                else {
                    console.log("other event :", event);
                }
            }, error => {
                console.log("Error", error['message']);
            });
        }
        else {
            //this.toastr.warning('other error', 'Warning');
            console.log("Error", 'other error');
        }
    }
    /**
     * @property {Function} onDismiss - dismiss image from select-to-upload
     * @param {MouseEvent} - click event
     * @returns {void}
     */
    onDismiss(event) {
        const imageTag = event.target['nextElementSibling'];
        for (let i in this.selectedUrls) {
            const index = Number(i);
            if (this.selectedUrls[index] == imageTag.getAttribute('src')) {
                this.selectedUrls.splice(index, 1);
                this.selectedFiles.splice(index, 1);
                break;
            }
        }
    }
    /**
     * @property {Function} onClear - clear the all images from select-to-upload
     * @returns {void}
     */
    onClear() {
        this.selectedUrls.length = 0;
        this.selectedFiles.length = 0;
    }
    /* ********************************* */
    /** //////////  setters   \\\\\\\\\\ */
    /* ********************************* */
    /**
     * @property {Function} onDismiss - dismiss image from select-to-upload
     * @param {MouseEvent} - click event
     * @returns {void}
     */
    setUploadedImages(event) {
    }
    /**
     * @property {Function} setUploadProgress - set upload progress
     * @param {number} - upload ratio count
     * @returns {void}
     */
    setUploadProgress(ratio) {
        this.uploadProgress = Math.round(ratio * 100);
        //console.log(" uploadProgress :", this.uploadProgress, ratio);
    }
    /**
     * @property {Function} setClearSelected - set pre view upload clear
     * @returns {void}
     */
    setClearSelected() {
        this.onClear();
    }
    /**
     * @property {Function} setUploadResponse - send upload responce to parent component
     * @param {number} - upload ratio count
     * @returns {void} - no returns
     */
    setUploadResponse(response) {
        this.onResponse.emit(response);
    }
    /**
     * @property {Function} setError - set error throw for developper
     * @param {string} - error message
     * @returns {void}
     */
    setError(error) {
        throw new Error(error);
    }
}
NgUploadImageComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-upload-image',
                template: "\r\n<div class=\"container\">\r\n\r\n    <!-- progress bar  -->\r\n    <div class=\"progress\" *ngIf=\"uploadProgress\"  style=\"margin: 5px 0px;\" >\r\n        <div class=\"wrapper\">\r\n            <div class=\"bar\"  [style.width]=\"uploadProgress+'%'\"   >{{uploadProgress}}%</div>\r\n        </div>\r\n    </div>\r\n\r\n    <!--control -->\r\n    <div clas=\"control\">\r\n        <input type=\"file\" name=\"imageUpload\" id=\"imageUpload\"  class=\"hide\" multiple  (change)=\"onSelect($event)\"/>\r\n        <label for=\"imageUpload\" class=\"button\" style=\"margin-right: 5px; margin-bottom: 5px;\">Select files</label>\r\n\r\n        <button *ngIf=\"selectedUrls.length\" type='button' class=\"button\" style=\"margin-right: 5px; margin-bottom: 5px;\" (click)=\"onUpload($event)\">Upload</button>\r\n        <button *ngIf=\"selectedUrls.length\" type='button' class=\"button\" style=\"margin-right: 5px; margin-bottom: 5px;\" (click)=\"onClear()\">Clear</button>\r\n    </div>\r\n\r\n    <!-- pre-view  -->\r\n    <div class=\"pre-view\">\r\n        <div class='img-box'  *ngFor='let url of selectedUrls'>\r\n            <!-- <div class=\"img-inner\"></div> -->\r\n            <div class=\"dismiss\" *ngIf=\"!uploadProgress\" (click)=\"onDismiss($event)\">&#10005;</div>\r\n            <img  class='image' [src]=\"url\"  [style.width.px]=\"preViewSize\" >\r\n        </div>\r\n    </div>\r\n\r\n</div>\r\n",
                styles: [".container{border:3px solid #f7f7f9}.hide{border:4px solid red;display:none}.control,.pre-view{display:flex;flex-wrap:wrap}.pre-view{flex-direction:row}.pre-view .img-box{position:relative}.pre-view .img-box .dismiss{color:red;cursor:pointer;font-size:18px;line-height:0;margin:10px;padding:10px;position:absolute;right:0;top:0}.pre-view .img-box .image{border:1px solid red;margin:5px 10px}.progress>.wrapper{background-color:#e9ecef;border-radius:4px;display:flex;font-size:12px;height:16px;overflow:hidden}.progress>.wrapper>.bar{display:flex;flex-direction:column;justify-content:center;transition:width .6s ease}.button,.progress>.wrapper>.bar{background-color:#007bff;color:#fff;text-align:center}.button{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;border:3px solid transparent;border-radius:4px;box-sizing:border-box!important;cursor:pointer;display:inline-block;font-size:16px;font-weight:400;line-height:1.5;padding:3px 12px;transition:background-color .2s ease-in-out;user-select:none;vertical-align:middle;white-space:nowrap}.button:focus{border:3px solid #66b0ff}.button:hover{background-color:#006fe6}.button:active{border:3px solid #66b0ff;box-shadow:0 0 4px 4px #007bff}.button-primary{background-color:red}.button-primary:focus{border:3px solid #f66}.button-primary:hover{background-color:#e60000}.button-primary:active{border:3px solid #f66;box-shadow:0 0 4px 4px red}.button-secondary{background-color:#00f}.button-secondary:focus{border:3px solid #66f}.button-secondary:hover{background-color:#0000e6}.button-secondary:active{border:3px solid #66f;box-shadow:0 0 4px 4px #00f}"]
            },] }
];
NgUploadImageComponent.ctorParameters = () => [
    { type: HttpClient }
];
NgUploadImageComponent.propDecorators = {
    validTypes: [{ type: Input }],
    maxUploadCount: [{ type: Input }],
    preViewSize: [{ type: Input }],
    apiUrl: [{ type: Input }],
    onResponse: [{ type: Output }]
};

class NgUploadImageModule {
}
NgUploadImageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgUploadImageComponent],
                imports: [
                    CommonModule
                ],
                exports: [NgUploadImageComponent]
            },] }
];

/*
 * Public API Surface of ng-upload-image
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgUploadImageComponent, NgUploadImageModule, NgUploadImageService };
//# sourceMappingURL=ng-upload-image.js.map
